package com.io.yutian.elementoriginlib.command;

import com.io.yutian.elementoriginlib.command.argument.Argument;
import com.io.yutian.elementoriginlib.command.argument.ArgumentType;
import com.io.yutian.elementoriginlib.command.interfaces.Command;
import com.io.yutian.elementoriginlib.command.interfaces.CommandArgument;
import com.io.yutian.elementoriginlib.command.interfaces.SubCommand;
import com.io.yutian.elementoriginlib.exception.command.CommandRegisterException;
import com.io.yutian.elementoriginlib.logger.Logger;

import java.lang.reflect.Method;
import java.lang.reflect.Parameter;
import java.util.*;

public class CommandEntity {

    private static final Logger LOGGER = Logger.getLogger(CommandEntity.class);

    private Object instance;

    private String command;
    private String permission;

    private List<CommandEntry> childrens = new ArrayList<>();

    public CommandEntity(Object instance, String command, String permission, List<CommandEntry> childrens) {
        this.instance = instance;
        this.command = command;
        this.permission = permission;
        this.childrens = childrens;
    }

    public Object getInstance() {
        return instance;
    }

    public void addChild(CommandEntry child) {
        childrens.add(child);
    }

    public String getCommand() {
        return command;
    }

    public String getPermission() {
        return permission;
    }

    public List<CommandEntry> getChildrens() {
        return childrens;
    }

    public static CommandEntity parseFromClass(Class clazz) {
        if (clazz == null) {
            LOGGER.warn("Class is null");
            return null;
        }
        if (!clazz.isAnnotationPresent(Command.class)) {
            LOGGER.warn("Class " + clazz.getName() + " is not annotated with @Command");
            return null;
        }

        Object instance;
        try {
            instance = clazz.getConstructor(null).newInstance();
        } catch (Exception e) {
            LOGGER.warn("无法实例化类 " + clazz);
            return null;
        }

        Command commandAnnotation = (Command) clazz.getAnnotation(Command.class);
        String command = commandAnnotation.value();
        String permission = commandAnnotation.permission();

        List<CommandEntry> allEntries = new ArrayList<>();
        Map<String, CommandEntry> pathToEntryMap = new HashMap<>();
        List<CommandEntry> rootEntries = new ArrayList<>();

        try {
            for (Method method : clazz.getDeclaredMethods()) {
                if (!method.isAnnotationPresent(SubCommand.class)) {
                    continue;
                }

                CommandEntry commandEntry = parseFromMethod(method);
                if (commandEntry != null) {
                    allEntries.add(commandEntry);

                    String path = commandEntry.getFullName();
                    String parentPath = getParentPath(path);

                    if (pathToEntryMap.containsKey(path)) {
                        LOGGER.warn("命令路径冲突: 子命令 '" + path + "' 已经定义, 不能再定义父命令 '" + parentPath + "'");
                        throw new CommandRegisterException("命令路径冲突: 子命令 '" + path + "' 已经定义, 不能再定义父命令 '" + parentPath + "'");
                    }

                    if (!parentPath.isEmpty() && pathToEntryMap.containsKey(parentPath)) {
                        LOGGER.warn("命令路径冲突: 子命令 '" + path + "' 已经定义, 不能再定义父命令 '" + parentPath + "'");
                        throw new CommandRegisterException("命令路径冲突: 子命令 '" + path + "' 已经定义, 不能再定义父命令 '" + parentPath + "'");
                    }

                    pathToEntryMap.put(commandEntry.getFullName(), commandEntry);
                }
            }

            for (CommandEntry entry : allEntries) {
                String fullPath = entry.getFullName();
                int lastDotIndex = fullPath.lastIndexOf('.');

                if (lastDotIndex == -1) {
                    rootEntries.add(entry);
                } else {
                    String parentPath = fullPath.substring(0, lastDotIndex);
                    CommandEntry parentEntry = pathToEntryMap.get(parentPath);

                    if (parentEntry == null) {
                        parentEntry = createParentNodes(pathToEntryMap, parentPath);
                        if (parentPath.indexOf('.') == -1) {
                            rootEntries.add(parentEntry);
                        }
                    }
                    parentEntry.getChildrens().add(entry);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }

        return new CommandEntity(instance, command, permission, rootEntries);
    }

    private static String getParentPath(String path) {
        int lastDotIndex = path.lastIndexOf('.');
        if (lastDotIndex == -1) {
            return "";
        }
        return path.substring(0, lastDotIndex);
    }

    private static CommandEntry createParentNodes(Map<String, CommandEntry> pathToEntryMap, String currentPath) {
        if (pathToEntryMap.containsKey(currentPath)) {
            return pathToEntryMap.get(currentPath);
        }

        int lastDotIndex = currentPath.lastIndexOf('.');
        CommandEntry parentEntry;

        if (lastDotIndex == -1) {
            parentEntry = new CommandEntry(currentPath, currentPath);
        } else {
            String grandParentPath = currentPath.substring(0, lastDotIndex);
            CommandEntry grandParentEntry = createParentNodes(pathToEntryMap, grandParentPath);
            parentEntry = new CommandEntry(currentPath, currentPath.substring(lastDotIndex + 1));
            grandParentEntry.getChildrens().add(parentEntry);
        }
        pathToEntryMap.put(currentPath, parentEntry);
        return parentEntry;
    }


    private static CommandEntry parseFromMethod(Method method) {
        SubCommand subCommand = method.getAnnotation(SubCommand.class);
        String path = subCommand.value();
        String permission = subCommand.permission();
        String[] senderRequireArray = subCommand.senderRequire();
        List<SenderRequire> senderRequireList = new ArrayList<>();
        for (String senderRequire : senderRequireArray) {
            SenderRequire senderRequire1 = SenderRequires.get(senderRequire);
            if (senderRequire1 == null) {
                LOGGER.warn("Sender require " + senderRequire + " is null");
                continue;
            }
            senderRequireList.add(SenderRequires.get(senderRequire));
        }

        String subName = path;

        if (path.contains(".")) {
            int lastDotIndex = path.lastIndexOf('.');
            if (lastDotIndex == -1 || lastDotIndex > path.length() - 1) {
                LOGGER.warn("Invalid command name '" + path + "'");
                return null;
            }
            subName = path.substring(path.lastIndexOf('.') + 1);
        }
        List<Argument> arguments = new ArrayList<>();
        for (Parameter parameter : method.getParameters()) {
            if (!parameter.isAnnotationPresent(CommandArgument.class)) {
                continue;
            }
            CommandArgument commandArgument = parameter.getAnnotation(CommandArgument.class);
            String name = commandArgument.name();
            String argumentName = name.isEmpty() ? parameter.getName() : name;
            boolean required = commandArgument.required();
            String suggestionType = commandArgument.suggestionType();
            Class<?> type = parameter.getType();
            ArgumentType argumentType = ArgumentType.get(parameter.getType());
            if (argumentType == null) {
                LOGGER.warn("Argument type " + parameter.getType().getName() + " is null");
                continue;
            }
            Argument argument = new Argument(argumentName, argumentType);
            if (!suggestionType.isEmpty()) {
                Suggest suggest = Suggests.getSuggest(suggestionType);
                if (suggest == null) {
                    LOGGER.warn("Suggest type " + suggestionType + " is null");
                } else {
                    argument.suggest(suggest);
                }
            }
            arguments.add(argument);
        }
        return new CommandEntry(path, subName, permission, senderRequireList, arguments, method);
    }

    @Override
    public String toString() {
        return "CommandEntity{" +
                "command='" + command + '\'' +
                ", permission='" + permission + '\'' +
                ", childrens=" + childrens +
                '}';
    }
}
